@@ -27,7 +27,6 @@
 #include <config.h>
 #endif
 
-#include <gconf/gconf-client.h>
 #include <glade/glade.h>
 #include <gnome-wm-manager.h>
 
@@ -42,7 +41,6 @@ typedef struct
         GtkWidget *radio;
 } MouseClickModifier;
 
-static GConfClient *gconf_client;
 static GladeXML *dialog;
 static GnomeWindowManager *current_wm; /* may be NULL */
 static GtkWidget *dialog_win;
@@ -51,7 +49,6 @@ static GtkWidget *autoraise_checkbutton;
 static GtkWidget *autoraise_delay_slider;
 static GtkWidget *autoraise_delay_hbox;
 static GtkWidget *double_click_titlebar_optionmenu;
-static GtkWidget *double_click_titlebar_hbox;
 static GtkWidget *alt_click_hbox;
 
 static GnomeWMSettings *settings;
@@ -430,8 +427,6 @@ main (int argc, char **argv)
 			    LIBGNOMEUI_MODULE, argc, argv,
 			    NULL);
 
-        gconf_client = gconf_client_get_default ();
-        
         gnome_wm_manager_init ();
 
         screen = gdk_display_get_default_screen (gdk_display_get_default ());
@@ -457,7 +452,6 @@ main (int argc, char **argv)
         autoraise_delay_slider = WID ("autoraise-delay-slider");
         autoraise_delay_hbox = WID ("autoraise-delay-hbox");
         double_click_titlebar_optionmenu = WID ("double-click-titlebar-optionmenu");
-        double_click_titlebar_hbox = WID ("double-click-titlebar-hbox");
         alt_click_hbox = WID ("alt-click-box");
 
         gtk_range_set_range (GTK_RANGE (autoraise_delay_slider),
@@ -535,7 +529,7 @@ fill_radio (GtkRadioButton     *group,
             MouseClickModifier *modifier)
 {
         modifier->radio =
-                gtk_radio_button_new_with_label_from_widget (group,
+                gtk_radio_button_new_with_mnemonic_from_widget (group,
                                                              modifier->name);
         gtk_box_pack_start (GTK_BOX (alt_click_hbox),
                             modifier->radio, FALSE, FALSE, 0);
@@ -645,32 +639,32 @@ reload_mouse_modifiers (void)
         i = 0;
 
         mouse_modifiers[i].number = i;
-        mouse_modifiers[i].name = g_strdup (_("Control"));
+        mouse_modifiers[i].name = g_strdup (_("C_ontrol"));
         mouse_modifiers[i].value = "Control";
         ++i;
 
         mouse_modifiers[i].number = i;
-        mouse_modifiers[i].name = g_strdup (_("Alt"));
+        mouse_modifiers[i].name = g_strdup (_("_Alt"));
         mouse_modifiers[i].value = "Alt";
         ++i;
 
         if (have_hyper) {
                 mouse_modifiers[i].number = i;
-                mouse_modifiers[i].name = g_strdup (_("Hyper"));
+                mouse_modifiers[i].name = g_strdup (_("H_yper"));
                 mouse_modifiers[i].value = "Hyper";
                 ++i;
         }
 
         if (have_super) {
                 mouse_modifiers[i].number = i;
-                mouse_modifiers[i].name = g_strdup (_("Super (or \"Windows logo\")"));
+                mouse_modifiers[i].name = g_strdup (_("S_uper (or \"Windows logo\")"));
                 mouse_modifiers[i].value = "Super";
                 ++i;
         }
 
         if (have_meta) {
                 mouse_modifiers[i].number = i;
-                mouse_modifiers[i].name = g_strdup (_("Meta"));
+                mouse_modifiers[i].name = g_strdup (_("_Meta"));
                 mouse_modifiers[i].value = "Meta";
                 ++i;
         }
